#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # 2A.i - Mesures de vitesse sur les dataframes
# 
# Lire un [dataframe](https://pandas.pydata.org/pandas-docs/stable/generated/pandas.DataFrame.html) avec un itérateur quand on ne connaît pas sa taille, lire un [array](https://docs.scipy.org/doc/numpy/reference/generated/numpy.array.html) avec un itérateur.

# In[1]:


from jyquickhelper import add_notebook_menu
add_notebook_menu()


# ## Création d'un dataframe à partir d'un itérateur
# 
# On cherche à créer un dataframe à partir d'un ensemble de lignes dont on ne connaît pas le nombre au moment où on créé le dataframe car on les reçoit sous la forme d'un itérateur ou un générateur.

# In[2]:


import random
def enumerate_row(nb=10000, n=10):
    for i in range(nb):
        # on retourne un tuple, les données sont
        # plus souvent recopiées car le type est immuable
        yield tuple(random.random() for k in range(n))
        # on retourne une liste, ces listes ne sont pas
        # recopiées en général, seule la liste qui les tient
        # l'est
        # yield list(random.random() for k in range(n))
        
list(enumerate_row(2))


# In[3]:


import pandas
nb, n = 10, 10
df = pandas.DataFrame(enumerate_row(nb=nb, n=n), columns=["c%d" % i for i in range(n)])
df.head()


# In[4]:


nb, n =100000, 10


# On compare plusieurs constructions :

# In[5]:


print(nb,n)
get_ipython().run_line_magic('timeit', 'pandas.DataFrame(enumerate_row(nb=nb,n=n), columns=["c%d" % i for i in range(n)])')


# In[6]:


print(nb,n)
get_ipython().run_line_magic('timeit', 'pandas.DataFrame(list(enumerate_row(nb=nb,n=n)), columns=["c%d" % i for i in range(n)])')


# On décompose :

# In[7]:


def cache():
    return list(enumerate_row(nb=nb,n=n))
print(nb,n)
get_ipython().run_line_magic('timeit', 'cache()')


# In[8]:


print(nb,n)
l = list(enumerate_row(nb=nb,n=n))
get_ipython().run_line_magic('timeit', 'pandas.DataFrame(l, columns=["c%d" % i for i in range(n)])')


# D'après ces temps, pandas convertit probablement l'itérateur en liste. On essaye de créer le dataframe vide, puis avec la méthode [from_records](http://pandas.pydata.org/pandas-docs/stable/dsintro.html#alternate-constructors).

# In[9]:


get_ipython().run_line_magic('timeit', 'pandas.DataFrame(columns=["c%d" % i for i in range(n)], index=range(n))')


# In[10]:


def create_df3():
    return pandas.DataFrame.from_records(enumerate_row(nb=nb,n=n), 
                                         columns=["c%d" % i for i in range(n)])

print(nb,n)
get_ipython().run_line_magic('timeit', 'create_df3()')


# ## Création d'un array à partir d'un itérateur
# 
# On cherche à créer un dataframe à partir d'un ensemble de lignes dont on ne connaît pas le nombre au moment où on créé le dataframe car on les reçoit sous la forme d'un itérateur ou un générateur. La documentation de la fonction [numpy.fromiter](http://docs.scipy.org/doc/numpy/reference/generated/numpy.fromiter.html#numpy.fromiter) est intéressante à ce sujet.

# In[11]:


def enumerate_row2(nb=10000, n=10):
    for i in range(nb):
        for k in range(n):
            yield random.random()
            
import numpy
nb,n = 100000, 10
# on précise la taille du tableau car cela évite à numpy d'agrandir le tableau
# au fur et à mesure, ceci ne fonctionne pas
print(nb,n)
m = numpy.fromiter(enumerate_row2(nb=nb,n=n), float, nb*n)
m.resize((nb,n))
m [:5,:]


# In[12]:


def create_array():
    m = numpy.fromiter(enumerate_row2(nb=nb,n=n), float, nb*n)
    m.resize((nb,n))    
    return m
print(nb,n)
get_ipython().run_line_magic('timeit', 'create_array()')


# In[13]:


def create_array2():
    m = list(enumerate_row(nb=nb,n=n))
    ml = numpy.array(m, float)
    return ml
print(nb,n)
get_ipython().run_line_magic('timeit', 'create_array2()')


# Et si on ne précise pas la taille du tableau créé avec la fonction ``fromiter`` :

# In[14]:


def create_array3():
    m = numpy.fromiter(enumerate_row2(nb=nb,n=n), float)
    m.resize((nb,n))    
    return m
print(nb,n)
get_ipython().run_line_magic('timeit', 'create_array3()')


# On retrouve des temps similaires que ceux obtenus avec une liste. En conclusion, pour créer un *array*, il vaut mieux :
# 
# * connaître la taille finale
# * éviter de créer une liste

# Pour finir, je recommande la lecture de [Enhancing Performance](http://pandas-docs.github.io/pandas-docs-travis/enhancingperf.html?highlight=dataframe) qui étudie différent scénari avec [cython](http://cython.org/), [eval](http://pandas.pydata.org/pandas-docs/stable/generated/pandas.DataFrame.eval.html), [numba](http://numba.pydata.org/).

# In[15]: